@@ -1,9 +1,28 @@
 package Moo;
 
 use Moo::_strictures;
-use Moo::_Utils;
+use Moo::_mro;
+use Moo::_Utils qw(
+  _getglob
+  _getstash
+  _install_coderef
+  _install_modifier
+  _load_module
+  _set_loaded
+  _unimport_coderefs
+);
+use Carp qw(croak);
+BEGIN {
+  our @CARP_NOT = qw(
+    Method::Generate::Constructor
+    Method::Generate::Accessor
+    Moo::sification
+    Moo::_Utils
+    Moo::Role
+  );
+}
 
-our $VERSION = '2.001000';
+our $VERSION = '2.002002';
 $VERSION = eval $VERSION;
 
 require Moo::sification;
@@ -26,7 +45,7 @@ sub import {
   warnings->import;
 
   if ($INC{'Role/Tiny.pm'} and Role::Tiny->is_role($target)) {
-    die "Cannot import Moo into a role";
+    croak "Cannot import Moo into a role";
   }
   $MAKERS{$target} ||= {};
   _install_tracked $target => extends => sub {
@@ -43,9 +62,8 @@ sub import {
     my $name_proto = shift;
     my @name_proto = ref $name_proto eq 'ARRAY' ? @$name_proto : $name_proto;
     if (@_ % 2 != 0) {
-      require Carp;
-      Carp::croak("Invalid options for " . join(', ', map "'$_'", @name_proto)
-        . " attribute(s): even number of arguments expected, got " . scalar @_)
+      croak "Invalid options for " . join(', ', map "'$_'", @name_proto)
+        . " attribute(s): even number of arguments expected, got " . scalar @_;
     }
     my %spec = @_;
     foreach my $name (@name_proto) {
@@ -62,7 +80,6 @@ sub import {
   };
   foreach my $type (qw(before after around)) {
     _install_tracked $target => $type => sub {
-      require Class::Method::Modifiers;
       _install_modifier($target, $type, @_);
       return;
     };
@@ -78,7 +95,7 @@ sub import {
       require Moo::Object; ('Moo::Object');
     } unless @{"${target}::ISA"};
   }
-  if ($INC{'Moo/HandleMoose.pm'}) {
+  if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
     Moo::HandleMoose::inject_fake_metaclass_for($target);
   }
 }
@@ -94,8 +111,7 @@ sub _set_superclasses {
   foreach my $superclass (@_) {
     _load_module($superclass);
     if ($INC{'Role/Tiny.pm'} && Role::Tiny->is_role($superclass)) {
-      require Carp;
-      Carp::croak("Can't extend role '$superclass'");
+      croak "Can't extend role '$superclass'";
     }
   }
   # Can't do *{...} = \@_ or 5.10.0's mro.pm stops seeing @ISA
@@ -117,7 +133,7 @@ sub _set_superclasses {
 
 sub _maybe_reset_handlemoose {
   my ($class, $target) = @_;
-  if ($INC{"Moo/HandleMoose.pm"}) {
+  if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
     Moo::HandleMoose::maybe_reinject_fake_metaclass_for($target);
   }
 }
@@ -162,10 +178,10 @@ sub _constructor_maker_for {
         '      if ($Moo::MAKERS{$class}) {'."\n"
         .'        if ($Moo::MAKERS{$class}{constructor}) {'."\n"
         .'          package '.$target.';'."\n"
-        .'          return $class->SUPER::new(@_);'."\n"
+        .'          return $invoker->SUPER::new(@_);'."\n"
         .'        }'."\n"
         .'        '.$class.'->_constructor_maker_for($class);'."\n"
-        .'        return $class->new(@_)'.";\n"
+        .'        return $invoker->new(@_)'.";\n"
         .'      } elsif ($INC{"Moose.pm"} and my $meta = Class::MOP::get_metaclass_by_name($class)) {'."\n"
         .'        return $meta->new_object('."\n"
         .'          $class->can("BUILDARGS") ? $class->BUILDARGS(@_)'."\n"
@@ -193,13 +209,13 @@ sub _constructor_maker_for {
           'do {'
           .'  my $args = $class->'.$inv.'BUILDARGS(@_);'
           .'  $args->{__no_BUILD__} = 1;'
-          .'  $class->'.$target.'::SUPER::new($args);'
+          .'  $invoker->'.$target.'::SUPER::new($args);'
           .'}'
         };
       }
       else {
         $construct_opts{construction_builder} = sub {
-          '$class->'.$target.'::SUPER::new('
+          '$invoker->'.$target.'::SUPER::new('
             .($target->can('FOREIGNBUILDARGS') ?
               '$class->FOREIGNBUILDARGS(@_)' : '@_')
             .')'
@@ -430,7 +446,7 @@ superclasses.
 Returns a L<Moose metaclass|Moose::Meta::Class> object for the class.  The
 metaclass will only be built on demand, loading Moose in the process.
 
-=head2 LIFECYCLE METHODS
+=head1 LIFECYCLE METHODS
 
 There are several methods that you can define in your class to control
 construction and destruction of objects.  They should be used rather than trying
