@@ -3,6 +3,7 @@ use Moo::_strictures;
 no warnings 'once';
 use Moo::_Utils qw(_getstash);
 use Sub::Quote qw(quotify);
+use Carp qw(croak);
 
 our %TYPE_MAP;
 
@@ -11,7 +12,7 @@ our $SETUP_DONE;
 sub import { return if $SETUP_DONE; inject_all(); $SETUP_DONE = 1; }
 
 sub inject_all {
-  die "Can't inflate Moose metaclass with Moo::sification disabled"
+  croak "Can't inflate Moose metaclass with Moo::sification disabled"
     if $Moo::sification::disabled;
   require Class::MOP;
   inject_fake_metaclass_for($_)
@@ -97,7 +98,7 @@ sub inject_real_metaclass_for {
   # needed to ensure the method body is stable and get things named
   $methods{$_} = Sub::Defer::undefer_sub($methods{$_})
     for
-      grep $_ ne 'new' && defined $methods{$_},
+      grep $_ ne 'new',
       keys %methods;
   my @attrs;
   {
@@ -131,7 +132,7 @@ sub inject_real_metaclass_for {
             my $type = $mapped->();
             unless ( Scalar::Util::blessed($type)
                 && $type->isa("Moose::Meta::TypeConstraint") ) {
-              die "error inflating attribute '$name' for package '$_[0]': "
+              croak "error inflating attribute '$name' for package '$_[0]': "
                 ."\$TYPE_MAP{$isa} did not return a valid type constraint'";
             }
             $coerce ? $type->create_child_type(name => $type->name) : $type;
@@ -180,7 +181,7 @@ sub inject_real_metaclass_for {
   }
   foreach my $meth_name (keys %methods) {
     my $meth_code = $methods{$meth_name};
-    $meta->add_method($meth_name, $meth_code) if $meth_code;
+    $meta->add_method($meth_name, $meth_code);
   }
 
   if ($am_role) {
